@@ -51,15 +51,15 @@ const mod_auth_st dhe_rsa_auth_struct = {
   _gnutls_gen_cert_client_certificate,
   gen_dhe_server_kx,
   _gnutls_gen_dh_common_client_kx,
-  _gnutls_gen_cert_client_cert_vrfy,	/* gen client cert vrfy */
-  _gnutls_gen_cert_server_cert_req,	/* server cert request */
+  _gnutls_gen_cert_client_cert_vrfy,    /* gen client cert vrfy */
+  _gnutls_gen_cert_server_cert_req,     /* server cert request */
 
   _gnutls_proc_cert_server_certificate,
   _gnutls_proc_cert_client_certificate,
   proc_dhe_server_kx,
   proc_dhe_client_kx,
-  _gnutls_proc_cert_client_cert_vrfy,	/* proc client cert vrfy */
-  _gnutls_proc_cert_cert_req	/* proc server cert request */
+  _gnutls_proc_cert_client_cert_vrfy,   /* proc client cert vrfy */
+  _gnutls_proc_cert_cert_req    /* proc server cert request */
 };
 
 const mod_auth_st dhe_dss_auth_struct = {
@@ -68,15 +68,15 @@ const mod_auth_st dhe_dss_auth_struct = {
   _gnutls_gen_cert_client_certificate,
   gen_dhe_server_kx,
   _gnutls_gen_dh_common_client_kx,
-  _gnutls_gen_cert_client_cert_vrfy,	/* gen client cert vrfy */
-  _gnutls_gen_cert_server_cert_req,	/* server cert request */
+  _gnutls_gen_cert_client_cert_vrfy,    /* gen client cert vrfy */
+  _gnutls_gen_cert_server_cert_req,     /* server cert request */
 
   _gnutls_proc_cert_server_certificate,
   _gnutls_proc_cert_client_certificate,
   proc_dhe_server_kx,
   proc_dhe_client_kx,
-  _gnutls_proc_cert_client_cert_vrfy,	/* proc client cert vrfy */
-  _gnutls_proc_cert_cert_req	/* proc server cert request */
+  _gnutls_proc_cert_client_cert_vrfy,   /* proc client cert vrfy */
+  _gnutls_proc_cert_cert_req    /* proc server cert request */
 };
 
 
@@ -89,7 +89,7 @@ gen_dhe_server_kx (gnutls_session_t session, opaque ** data)
   gnutls_cert *apr_cert_list;
   gnutls_privkey_t apr_pkey;
   int apr_cert_list_length;
-  gnutls_datum_t signature, ddata;
+  gnutls_datum_t signature = { NULL, 0 }, ddata;
   gnutls_certificate_credentials_t cred;
   gnutls_dh_params_t dh_params;
   gnutls_sign_algorithm_t sign_algo;
@@ -106,7 +106,7 @@ gen_dhe_server_kx (gnutls_session_t session, opaque ** data)
   /* find the appropriate certificate */
   if ((ret =
        _gnutls_get_selected_cert (session, &apr_cert_list,
-				  &apr_cert_list_length, &apr_pkey)) < 0)
+                                  &apr_cert_list_length, &apr_pkey)) < 0)
     {
       gnutls_assert ();
       return ret;
@@ -125,7 +125,7 @@ gen_dhe_server_kx (gnutls_session_t session, opaque ** data)
   g = mpis[1];
 
   if ((ret = _gnutls_auth_info_set (session, GNUTLS_CRD_CERTIFICATE,
-				    sizeof (cert_auth_info_st), 0)) < 0)
+                                    sizeof (cert_auth_info_st), 0)) < 0)
     {
       gnutls_assert ();
       return ret;
@@ -149,43 +149,49 @@ gen_dhe_server_kx (gnutls_session_t session, opaque ** data)
   if (apr_cert_list_length > 0)
     {
       if ((ret =
-	   _gnutls_handshake_sign_data (session, &apr_cert_list[0],
-					apr_pkey, &ddata, &signature,
-					&sign_algo)) < 0)
-	{
-	  gnutls_assert ();
-	  gnutls_free (*data);
-	  return ret;
-	}
+           _gnutls_handshake_sign_data (session, &apr_cert_list[0],
+                                        apr_pkey, &ddata, &signature,
+                                        &sign_algo)) < 0)
+        {
+          gnutls_assert ();
+          goto cleanup;
+        }
     }
   else
     {
       gnutls_assert ();
-      return data_size;		/* do not put a signature - ILLEGAL! */
+      ret = data_size;         /* do not put a signature - ILLEGAL! */
+      goto cleanup;
     }
 
   *data = gnutls_realloc_fast (*data, data_size + signature.size + 4);
   if (*data == NULL)
     {
-      _gnutls_free_datum (&signature);
       gnutls_assert ();
-      return GNUTLS_E_MEMORY_ERROR;
+      ret = GNUTLS_E_MEMORY_ERROR;
+      goto cleanup;
     }
 
   if (_gnutls_version_has_selectable_sighash (ver))
     {
-      sign_algorithm_st aid;
+      const sign_algorithm_st *aid;
 
       if (sign_algo == GNUTLS_SIGN_UNKNOWN)
-	{
-	  _gnutls_free_datum (&signature);
-	  gnutls_assert ();
-	  return GNUTLS_E_UNKNOWN_PK_ALGORITHM;
-	}
+        {
+          ret = GNUTLS_E_UNKNOWN_ALGORITHM;
+          goto cleanup;
+        }
 
       aid = _gnutls_sign_to_tls_aid (sign_algo);
-      (*data)[data_size++] = aid.hash_algorithm;
-      (*data)[data_size++] = aid.sign_algorithm;
+      if (aid == NULL)
+        {
+          gnutls_assert();
+          ret = GNUTLS_E_UNKNOWN_ALGORITHM;
+          goto cleanup;
+        }
+      
+      (*data)[data_size++] = aid->hash_algorithm;
+      (*data)[data_size++] = aid->sign_algorithm;
     }
 
   _gnutls_write_datum16 (&(*data)[data_size], signature);
@@ -194,11 +200,17 @@ gen_dhe_server_kx (gnutls_session_t session, opaque ** data)
   _gnutls_free_datum (&signature);
 
   return data_size;
+
+cleanup:
+  _gnutls_free_datum (&signature);
+  gnutls_free(*data);
+  return ret;
+
 }
 
 static int
 proc_dhe_server_kx (gnutls_session_t session, opaque * data,
-		    size_t _data_size)
+                    size_t _data_size)
 {
   int sigsize;
   opaque *sigdata;
@@ -240,10 +252,10 @@ proc_dhe_server_kx (gnutls_session_t session, opaque * data,
       aid.sign_algorithm = *sigdata++;
       sign_algo = _gnutls_tls_aid_to_sign (&aid);
       if (sign_algo == GNUTLS_SIGN_UNKNOWN)
-	{
-	  gnutls_assert ();
-	  return GNUTLS_E_UNSUPPORTED_SIGNATURE_ALGORITHM;
-	}
+        {
+          gnutls_assert ();
+          return GNUTLS_E_UNSUPPORTED_SIGNATURE_ALGORITHM;
+        }
     }
   DECR_LEN (data_size, 2);
   sigsize = _gnutls_read_uint16 (sigdata);
@@ -255,8 +267,8 @@ proc_dhe_server_kx (gnutls_session_t session, opaque * data,
 
   if ((ret =
        _gnutls_get_auth_info_gcert (&peer_cert,
-				    session->security_parameters.cert_type,
-				    info, CERT_NO_COPY)) < 0)
+                                    session->security_parameters.cert_type,
+                                    info, CERT_NO_COPY)) < 0)
     {
       gnutls_assert ();
       return ret;
@@ -264,7 +276,7 @@ proc_dhe_server_kx (gnutls_session_t session, opaque * data,
 
   ret =
     _gnutls_handshake_verify_data (session, &peer_cert, &vparams, &signature,
-				   sign_algo);
+                                   sign_algo);
 
   _gnutls_gcert_deinit (&peer_cert);
   if (ret < 0)
@@ -280,7 +292,7 @@ proc_dhe_server_kx (gnutls_session_t session, opaque * data,
 
 static int
 proc_dhe_client_kx (gnutls_session_t session, opaque * data,
-		    size_t _data_size)
+                    size_t _data_size)
 {
   gnutls_certificate_credentials_t cred;
   int ret;
